@@ -34,7 +34,9 @@ use crate::errors::{MethodNotAllowedInWith, PotentiallySlowLoop};
 use crate::re;
 use crate::symbols::{Symbol, SymbolLookup, SymbolTable};
 use crate::types::Value::Const;
-use crate::types::{Map, Regexp, StringConstraint, Type, TypeValue};
+use crate::types::{
+    IntegerConstraint, Map, Regexp, StringConstraint, Type, TypeValue,
+};
 use crate::warnings::UnsatisfiableExpression;
 
 /// How many patterns a rule can have. If a rule has more than this number of
@@ -507,7 +509,7 @@ fn expr_from_ast(
                 match (lhs_expr.type_value(), rhs_expr.type_value()) {
                     (
                         TypeValue::Bool { .. },
-                        TypeValue::Integer { value: Const(0) },
+                        TypeValue::Integer { value: Const(0), .. },
                     ) => Some((
                         ctx.ir.not(lhs),
                         format!(
@@ -518,7 +520,7 @@ fn expr_from_ast(
                         ),
                     )),
                     (
-                        TypeValue::Integer { value: Const(0) },
+                        TypeValue::Integer { value: Const(0), .. },
                         TypeValue::Bool { .. },
                     ) => Some((
                         ctx.ir.not(rhs),
@@ -531,7 +533,7 @@ fn expr_from_ast(
                     )),
                     (
                         TypeValue::Bool { .. },
-                        TypeValue::Integer { value: Const(1) },
+                        TypeValue::Integer { value: Const(1), .. },
                     ) => Some((
                         lhs,
                         ctx.report_builder.get_snippet(
@@ -539,7 +541,7 @@ fn expr_from_ast(
                         ),
                     )),
                     (
-                        TypeValue::Integer { value: Const(1) },
+                        TypeValue::Integer { value: Const(1), .. },
                         TypeValue::Bool { .. },
                     ) => Some((
                         rhs,
@@ -637,6 +639,20 @@ fn expr_from_ast(
                 }
             }
 
+            // If the field is deprecated raise the appropriate warning.
+            if let Symbol::Field { deprecation_msg: Some(ref msg), .. } =
+                symbol
+            {
+                ctx.warnings.add(|| {
+                    warnings::DeprecatedField::build(
+                        ctx.report_builder,
+                        ident.name.to_string(),
+                        ctx.report_builder.span_to_code_loc(ident.span()),
+                        msg.clone(),
+                    )
+                })
+            }
+
             ctx.ir.ident(symbol)
         }
 
@@ -820,6 +836,8 @@ fn expr_from_ast(
         ast::Expr::Lookup(expr) => {
             let primary = expr_from_ast(ctx, &expr.primary)?;
 
+            ctx.one_shot_symbol_table = None;
+
             match ctx.ir.get(primary).type_value() {
                 TypeValue::Array(array) => {
                     let index =
@@ -1068,16 +1086,16 @@ fn of_expr_from_ast(
             // `<expr> of <items> at <expr>: the warning is raised if <expr> is
             // 2 or more.
             Quantifier::Expr(expr) => match ctx.ir.get(*expr).type_value() {
-                TypeValue::Integer { value: Const(value) } => value >= 2,
+                TypeValue::Integer { value: Const(value), .. } => value >= 2,
                 _ => false,
             },
             // `<expr>% of <items> at <expr>: the warning is raised if the
             // <expr> percent of the items is 2 or more.
             Quantifier::Percentage(expr) => {
                 match ctx.ir.get(*expr).type_value() {
-                    TypeValue::Integer { value: Const(percentage) } => {
-                        items.len() as f64 * percentage as f64 / 100.0 >= 2.0
-                    }
+                    TypeValue::Integer {
+                        value: Const(percentage), ..
+                    } => items.len() as f64 * percentage as f64 / 100.0 >= 2.0,
                     _ => false,
                 }
             }
@@ -1180,10 +1198,8 @@ fn is_potentially_large_range(ctx: &CompileContext, range: &Range) -> bool {
             |node| matches!(node, Expr::Filesize | Expr::PatternCount { .. }),
             // Don't traverse the arguments of `math.min`.
             |node| {
-                if let Expr::FuncCall(f) = node {
-                    f.func.signatures().iter().any(|signature| {
-                        signature.mangled_name.as_str().eq("math.min@ii@i")
-                    })
+                if let Expr::FuncCall(func) = node {
+                    func.signature.mangled_name.as_str().eq("math.min@ii@i")
                 } else {
                     false
                 }
@@ -1357,7 +1373,7 @@ fn with_expr_from_ast(
         // a variable for it, but add it to the symbol table. Methods are not
         // allowed though.
         if let TypeValue::Func(func) = &type_value {
-            if func.method_of().is_some() {
+            if func.is_method() {
                 return Err(MethodNotAllowedInWith::build(
                     ctx.report_builder,
                     ctx.report_builder
@@ -1462,8 +1478,8 @@ fn range_from_ast(
     // variables, for example) we can't raise an error at compile time, but it
     // will be handled at scan time.
     if let (
-        TypeValue::Integer { value: Const(lower_bound) },
-        TypeValue::Integer { value: Const(upper_bound) },
+        TypeValue::Integer { value: Const(lower_bound), .. },
+        TypeValue::Integer { value: Const(upper_bound), .. },
     ) = (
         ctx.ir.get(lower_bound).type_value(),
         ctx.ir.get(upper_bound).type_value(),
@@ -1493,7 +1509,7 @@ fn non_negative_integer_from_ast(
     check_type(ctx, expr, span.clone(), &[Type::Integer])?;
 
     let type_value = ctx.ir.get(expr).type_value();
-    if let TypeValue::Integer { value: Const(value) } = type_value {
+    if let TypeValue::Integer { value: Const(value), .. } = type_value {
         if value < 0 {
             return Err(UnexpectedNegativeNumber::build(
                 ctx.report_builder,
@@ -1519,7 +1535,7 @@ fn integer_in_range_from_ast(
     // the given range.
     let type_value = ctx.ir.get(expr).type_value();
 
-    if let TypeValue::Integer { value: Const(value) } = type_value {
+    if let TypeValue::Integer { value: Const(value), .. } = type_value {
         if !range.contains(&value) {
             return Err(NumberOutOfRange::build(
                 ctx.report_builder,
@@ -1656,12 +1672,6 @@ fn func_call_from_ast(
         }
     };
 
-    // The object is necessary only when this is a method call, if this
-    // is a function call no object is required.
-    if func.method_of().is_none() {
-        object = None
-    }
-
     let args = func_call
         .args
         .iter()
@@ -1676,19 +1686,21 @@ fn func_call_from_ast(
 
     // Determine if any of the signatures for the called function matches
     // the provided arguments.
-    for (i, signature) in func.signatures().iter().enumerate() {
+    for signature in func.signatures().iter() {
         // If the function is actually a method, the first argument is always
         // the type the method belongs to (i.e: the self pointer). This
         // argument appears in the function's signature, but is not expected
         // to appear among the arguments in the call statement.
-        let expected_arg_types: Vec<Type> = if func.method_of().is_some() {
+
+        let expected_arg_types: Vec<Type> = if signature.method_of().is_some()
+        {
             signature.args.iter().skip(1).map(|arg| arg.ty()).collect()
         } else {
             signature.args.iter().map(|arg| arg.ty()).collect()
         };
 
         if arg_types == expected_arg_types {
-            matching_signature = Some((i, signature.result.clone()));
+            matching_signature = Some(signature);
             break;
         }
 
@@ -1720,9 +1732,15 @@ fn func_call_from_ast(
         ));
     }
 
-    let (signature_index, type_value) = matching_signature.unwrap();
+    let matching_signature = matching_signature.unwrap();
 
-    Ok(ctx.ir.func_call(object, args, func, type_value, signature_index))
+    // The object is necessary only when this is a method call, if this
+    // is a function call no object is required.
+    if matching_signature.method_of().is_none() {
+        object = None
+    }
+
+    Ok(ctx.ir.func_call(object, args, matching_signature.clone()))
 }
 
 fn matches_expr_from_ast(
@@ -2153,19 +2171,7 @@ gen_binary_op!(
     shl,
     Type::Integer,
     Type::Integer,
-    Some(|ctx, _lhs, rhs, _lhs_span, rhs_span| {
-        if let TypeValue::Integer { value: Const(value) } =
-            ctx.ir.get(rhs).type_value()
-        {
-            if value < 0 {
-                return Err(UnexpectedNegativeNumber::build(
-                    ctx.report_builder,
-                    ctx.report_builder.span_to_code_loc(rhs_span),
-                ));
-            }
-        }
-        Ok(())
-    })
+    Some(shx_check)
 );
 
 gen_binary_op!(
@@ -2173,19 +2179,7 @@ gen_binary_op!(
     shr,
     Type::Integer,
     Type::Integer,
-    Some(|ctx, _lhs, rhs, _lhs_span, rhs_span| {
-        if let TypeValue::Integer { value: Const(value) } =
-            ctx.ir.get(rhs).type_value()
-        {
-            if value < 0 {
-                return Err(UnexpectedNegativeNumber::build(
-                    ctx.report_builder,
-                    ctx.report_builder.span_to_code_loc(rhs_span),
-                ));
-            }
-        }
-        Ok(())
-    })
+    Some(shx_check)
 );
 
 gen_unary_op!(bitwise_not_expr_from_ast, bitwise_not, Type::Integer, None);
@@ -2222,84 +2216,7 @@ gen_binary_op!(
     // Integers can be compared with floats, but strings can be
     // compared only with another string.
     Type::Integer | Type::Float,
-    Some(|ctx, lhs, rhs, lhs_span, rhs_span| {
-        let lhs = ctx.ir.get(lhs).type_value();
-        let rhs = ctx.ir.get(rhs).type_value();
-
-        let (
-            const_string,
-            const_string_span,
-            constraints,
-            constrained_string_span,
-        ) = match (lhs, rhs, lhs_span, rhs_span) {
-            (
-                TypeValue::String { value: Const(const_string), .. },
-                TypeValue::String { constraints: Some(constraints), .. },
-                const_string_span,
-                constrained_string_span,
-            )
-            | (
-                TypeValue::String { constraints: Some(constraints), .. },
-                TypeValue::String { value: Const(const_string), .. },
-                constrained_string_span,
-                const_string_span,
-            ) => (
-                const_string,
-                const_string_span,
-                constraints,
-                constrained_string_span,
-            ),
-            _ => return Ok(()),
-        };
-
-        for constraint in constraints {
-            match constraint {
-                StringConstraint::Lowercase
-                    if const_string.chars().any(|c| c.is_uppercase()) =>
-                {
-                    ctx.warnings.add(|| {
-                        UnsatisfiableExpression::build(
-                            ctx.report_builder,
-                            "this is a lowercase string".to_string(),
-                            "this contains uppercase characters".to_string(),
-                            ctx.report_builder.span_to_code_loc(
-                                constrained_string_span.clone()
-                            ),
-                            ctx.report_builder.span_to_code_loc(
-                                const_string_span.clone()
-                            ),
-                            Some(
-                                "a lowercase string can't be equal to a string containing uppercase characters"
-                                .to_string())
-                        )
-                    });
-                }
-                StringConstraint::ExactLength(n)
-                    if const_string.len() != n =>
-                {
-                    ctx.warnings.add(|| {
-                        UnsatisfiableExpression::build(
-                            ctx.report_builder,
-                            format!("the length of this string is {}", n),
-                            format!(
-                                "the length of this string is {}",
-                                const_string.len()
-                            ),
-                            ctx.report_builder.span_to_code_loc(
-                                constrained_string_span.clone(),
-                            ),
-                            ctx.report_builder
-                                .span_to_code_loc(const_string_span.clone()),
-                            None,
-                        )
-                    });
-                }
-                _ => {}
-            }
-        }
-
-        Ok(())
-    })
+    Some(eq_check)
 );
 
 gen_binary_op!(
@@ -2364,3 +2281,168 @@ gen_string_op!(istartswith_expr_from_ast, istarts_with);
 gen_string_op!(endswith_expr_from_ast, ends_with);
 gen_string_op!(iendswith_expr_from_ast, iends_with);
 gen_string_op!(iequals_expr_from_ast, iequals);
+
+/// Checks the operands for the == operation.
+fn eq_check(
+    ctx: &mut CompileContext,
+    lhs: ExprId,
+    rhs: ExprId,
+    lhs_span: Span,
+    rhs_span: Span,
+) -> Result<(), CompileError> {
+    let lhs = ctx.ir.get(lhs).type_value();
+    let rhs = ctx.ir.get(rhs).type_value();
+
+    let check_string_constraints =
+        |ctx: &mut CompileContext,
+         const_string: Rc<BString>,
+         const_string_span: Span,
+         constraints: Vec<StringConstraint>,
+         constrained_string_span: Span| {
+            for constraint in constraints {
+                match constraint {
+                    StringConstraint::Lowercase
+                        if const_string.chars().any(|c| c.is_uppercase()) =>
+                    {
+                        ctx.warnings.add(|| {
+                            UnsatisfiableExpression::build(
+                                ctx.report_builder,
+                                "this is a lowercase string".to_string(),
+                                "this contains uppercase characters".to_string(),
+                                ctx.report_builder.span_to_code_loc(
+                                    constrained_string_span.clone()
+                                ),
+                                ctx.report_builder.span_to_code_loc(
+                                    const_string_span.clone()
+                                ),
+                                Some(
+                                    "a lowercase string can't be equal to a string containing uppercase characters"
+                                        .to_string())
+                            )
+                        });
+                    }
+                    StringConstraint::ExactLength(n)
+                        if const_string.len() != n =>
+                    {
+                        ctx.warnings.add(|| {
+                            UnsatisfiableExpression::build(
+                                ctx.report_builder,
+                                format!("the length of this string is {}", n),
+                                format!(
+                                    "the length of this string is {}",
+                                    const_string.len()
+                                ),
+                                ctx.report_builder.span_to_code_loc(
+                                    constrained_string_span.clone(),
+                                ),
+                                ctx.report_builder.span_to_code_loc(
+                                    const_string_span.clone(),
+                                ),
+                                None,
+                            )
+                        });
+                    }
+                    _ => {}
+                }
+            }
+        };
+
+    let check_integer_constraints =
+        |ctx: &mut CompileContext,
+         const_integer: i64,
+         const_integer_span: Span,
+         constraints: Vec<IntegerConstraint>,
+         constrained_integer_span: Span| {
+            for constraint in constraints {
+                match constraint {
+                    IntegerConstraint::Range(min, max)
+                        if !(min..=max).contains(&const_integer) =>
+                    {
+                        ctx.warnings.add(|| {
+                                UnsatisfiableExpression::build(
+                                    ctx.report_builder,
+                                    format!(
+                                        "this expression is an integer in the range [{min},{max}]",
+                                    ),
+                                    format!(
+                                        "this integer is outside the range [{min},{max}]",
+                                    ),
+                                    ctx.report_builder.span_to_code_loc(
+                                        constrained_integer_span.clone(),
+                                    ),
+                                    ctx.report_builder.span_to_code_loc(
+                                        const_integer_span.clone(),
+                                    ),
+                                    None,
+                                )
+                            });
+                    }
+
+                    _ => {}
+                }
+            }
+        };
+
+    match (lhs, rhs, lhs_span, rhs_span) {
+        (
+            TypeValue::String { value: Const(const_string), .. },
+            TypeValue::String { constraints: Some(constraints), .. },
+            const_string_span,
+            constrained_string_span,
+        )
+        | (
+            TypeValue::String { constraints: Some(constraints), .. },
+            TypeValue::String { value: Const(const_string), .. },
+            constrained_string_span,
+            const_string_span,
+        ) => check_string_constraints(
+            ctx,
+            const_string,
+            const_string_span,
+            constraints,
+            constrained_string_span,
+        ),
+        (
+            TypeValue::Integer { value: Const(const_integer), .. },
+            TypeValue::Integer { constraints: Some(constraints), .. },
+            const_integer_span,
+            constrained_integer_span,
+        )
+        | (
+            TypeValue::Integer { constraints: Some(constraints), .. },
+            TypeValue::Integer { value: Const(const_integer), .. },
+            constrained_integer_span,
+            const_integer_span,
+        ) => check_integer_constraints(
+            ctx,
+            const_integer,
+            const_integer_span,
+            constraints,
+            constrained_integer_span,
+        ),
+        _ => {}
+    };
+
+    Ok(())
+}
+
+/// Checks the operands for shift-left and shift-right operations.
+fn shx_check(
+    ctx: &mut CompileContext,
+    _lhs: ExprId,
+    rhs: ExprId,
+    _lhs_span: Span,
+    rhs_span: Span,
+) -> Result<(), CompileError> {
+    if let TypeValue::Integer { value: Const(value), .. } =
+        ctx.ir.get(rhs).type_value()
+    {
+        if value < 0 {
+            return Err(UnexpectedNegativeNumber::build(
+                ctx.report_builder,
+                ctx.report_builder.span_to_code_loc(rhs_span),
+            ));
+        }
+    }
+    Ok(())
+}
