@@ -99,10 +99,12 @@ use crate::types::{
     Array, Func, FuncSignature, Map, Struct, TypeValue, Value,
 };
 use crate::wasm;
+use crate::wasm::integer::RangedInteger;
 use crate::wasm::string::RuntimeString;
 use crate::wasm::string::String as _;
 
 pub(crate) mod builder;
+pub(crate) mod integer;
 pub(crate) mod string;
 
 /// Maximum number of variables.
@@ -225,13 +227,9 @@ impl WasmExport {
     /// fn some_method(...) { ... }
     /// ```
     pub fn get_methods(type_name: &str) -> FxHashMap<&'static str, Func> {
-        let mut methods = WasmExport::get_functions(|export| {
+        WasmExport::get_functions(|export| {
             export.method_of.is_some_and(|name| name == type_name)
-        });
-        for (_, func) in methods.iter_mut() {
-            func.make_method_of(type_name)
-        }
-        methods
+        })
     }
 }
 
@@ -436,6 +434,16 @@ impl WasmResult for i64 {
     }
 }
 
+impl<const MIN: i64, const MAX: i64> WasmResult for RangedInteger<MIN, MAX> {
+    fn values(self, _: &mut ScanContext) -> WasmResultArray<ValRaw> {
+        smallvec![ValRaw::i64(self.value())]
+    }
+
+    fn types() -> WasmResultArray<wasmtime::ValType> {
+        smallvec![wasmtime::ValType::I64]
+    }
+}
+
 impl WasmResult for f32 {
     fn values(self, _: &mut ScanContext) -> WasmResultArray<ValRaw> {
         smallvec![ValRaw::f32(f32::to_bits(self))]
@@ -925,7 +933,7 @@ pub(crate) fn pat_offset(
 }
 
 /// Called from WASM to obtain the length of an array.
-#[wasm_export]
+#[wasm_export(name = "len", method_of = "Array")]
 pub(crate) fn array_len(
     _: &mut Caller<'_, ScanContext>,
     array: Rc<Array>,
@@ -934,7 +942,7 @@ pub(crate) fn array_len(
 }
 
 /// Called from WASM to obtain the length of a map.
-#[wasm_export]
+#[wasm_export(name = "len", method_of = "Map")]
 pub(crate) fn map_len(_: &mut Caller<'_, ScanContext>, map: Rc<Map>) -> i64 {
     map.len() as i64
 }
@@ -1074,7 +1082,7 @@ macro_rules! gen_lookup_fn {
             structure: Option<Rc<Struct>>,
             num_lookup_indexes: i32,
         ) -> Option<$return_type> {
-            if let $type { value } =
+            if let $type { value, .. } =
                 lookup_field(caller, structure, num_lookup_indexes)
             {
                 value.extract().cloned()
@@ -1467,12 +1475,12 @@ pub(crate) fn str_matches(
 }
 
 macro_rules! gen_xint_fn {
-    ($name:ident, $return_type:ty, $from_fn:ident) => {
+    ($name:ident, $return_type:ty, $from_fn:ident, $min:expr, $max:expr) => {
         #[wasm_export(public = true)]
         pub(crate) fn $name(
             caller: &mut Caller<'_, ScanContext>,
             offset: i64,
-        ) -> Option<i64> {
+        ) -> Option<RangedInteger<$min, $max>> {
             let offset = usize::try_from(offset).ok()?;
             caller
                 .data()
@@ -1481,20 +1489,21 @@ macro_rules! gen_xint_fn {
                 .map(|bytes| {
                     <$return_type>::$from_fn(bytes.try_into().unwrap()) as i64
                 })
+                .map(|i| RangedInteger::<$min, $max>::new(i))
         }
     };
 }
 
-gen_xint_fn!(uint8, u8, from_le_bytes);
-gen_xint_fn!(uint16, u16, from_le_bytes);
-gen_xint_fn!(uint32, u32, from_le_bytes);
-gen_xint_fn!(uint8be, u8, from_be_bytes);
-gen_xint_fn!(uint16be, u16, from_be_bytes);
-gen_xint_fn!(uint32be, u32, from_be_bytes);
+gen_xint_fn!(uint8, u8, from_le_bytes, 0, 255);
+gen_xint_fn!(uint16, u16, from_le_bytes, 0, 65_535);
+gen_xint_fn!(uint32, u32, from_le_bytes, 0, 4_294_967_295);
+gen_xint_fn!(uint8be, u8, from_be_bytes, 0, 255);
+gen_xint_fn!(uint16be, u16, from_be_bytes, 0, 65_535);
+gen_xint_fn!(uint32be, u32, from_be_bytes, 0, 4_294_967_295);
 
-gen_xint_fn!(int8, i8, from_le_bytes);
-gen_xint_fn!(int16, i16, from_le_bytes);
-gen_xint_fn!(int32, i32, from_le_bytes);
-gen_xint_fn!(int8be, i8, from_be_bytes);
-gen_xint_fn!(int16be, i16, from_be_bytes);
-gen_xint_fn!(int32be, i32, from_be_bytes);
+gen_xint_fn!(int8, i8, from_le_bytes, -128, 127);
+gen_xint_fn!(int16, i16, from_le_bytes, -32_768, 32_767);
+gen_xint_fn!(int32, i32, from_le_bytes, -2_147_483_648, 2_147_483_647);
+gen_xint_fn!(int8be, i8, from_be_bytes, -128, 127);
+gen_xint_fn!(int16be, i16, from_be_bytes, -32_768, 32_767);
+gen_xint_fn!(int32be, i32, from_be_bytes, -2_147_483_648, 2_147_483_647);
